==> awsmobile-cli-1.0.21.tgz: package/lib/build-frontend.js <==
/* 
 * Copyright 2017-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 *
 *     http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
*/
"use strict";
const path = require('path')
const fs = require('fs-extra')
const ora = require('ora')
const moment = require('moment')
const chalk = require('chalk')
const _ = require('lodash')

const { spawn } = require('child_process')

const projectInfoManager = require('./project-info-manager.js')
const pathManager = require('./utils/awsmobilejs-path-manager.js')
const awsmobileJSConstant = require('./utils/awsmobilejs-constant.js')
const dfops = require('./utils/directory-file-ops.js')
const opsProject = require('./backend-operations/ops-project.js')

let _projectInfo

function build(callback) {
    _projectInfo = projectInfoManager.getProjectInfo()
    if(_projectInfo){
        runNpmInstall(function(){
            runBuildCommand(callback)
        })
    }
}

function checkNpmInstall(callback){
    _projectInfo = projectInfoManager.getProjectInfo()
    if(_projectInfo){
        runNpmInstall(callback)
    }
}

function runNpmInstall(callback){
    if(isNpmInstallNeeded()){
        console.log()
        console.log('executing npm install ...')
        let npm = /^win/.test(process.platform) ? 'npm.cmd' : 'npm'
        let npmInstall = spawn(npm, ['install'], {cwd: process.cwd(), env: process.env, stdio: 'inherit'})
        npmInstall.on('exit', function(code) {
            console.log('npm install exited with code ' + code.toString())
            if(code == 0){
                _projectInfo.LastNPMInstallTime = moment().format(awsmobileJSConstant.DateTimeFormatString) 
                projectInfoManager.setProjectInfo(_projectInfo)
                if(callback){
                    callback()
                }
            }
        })
        npmInstall.on('error', (err) => {
            console.log(chalk.red('npm install execution error'))
            console.log(err)
        })
    }else if(callback){
        callback()
    }
}

function runBuildCommand(callback){
    if(isNewBuildNeeded() && _projectInfo.BuildCommand && _projectInfo.BuildCommand.length > 0){
        console.log()
        console.log('executing frontend build command ...')

        let args = _projectInfo.BuildCommand.split(/\s+/)
        let command = args[0]
        args = args.slice(1)
        
        let buildExecution = spawn(command, args, {cwd: process.cwd(), env: process.env, stdio: 'inherit'})
        buildExecution.on('exit', function(code) {
            console.log('build command exited with code ' + code.toString())
            if(code == 0){
                _projectInfo.FrontendLastBuildTime = moment().format(awsmobileJSConstant.DateTimeFormatString) 
                projectInfoManager.setProjectInfo(_projectInfo)
                if(callback){
                    callback()
                }
            }
        })
        buildExecution.on('error', (err) => {
            console.log(chalk.red('frontend build command execution error'))
            console.log(err)
        })
    }else{
        if(callback){
            callback()
        }
    }
}

function isNpmInstallNeeded(){
    let result = false
    
    let packageJsonFilePath = path.normalize(path.join(_projectInfo.ProjectPath, 'package.json'))
    if(fs.existsSync(packageJsonFilePath)){
        let npm_modules_DirPath = path.normalize(path.join(_projectInfo.ProjectPath, 'node_modules'))
        let packageInfo = JSON.parse(fs.readFileSync(packageJsonFilePath, 'utf8'))
        if(packageInfo){

            let hasDependency = (packageInfo.dependencies &&  ! _.isEmpty(packageInfo.dependencies)) ||
                                (packageInfo.devDependencies &&  !_.isEmpty(packageInfo.devDependencies))
            if(hasDependency){
                if(!fs.existsSync(npm_modules_DirPath)){
                    result = true
                }else{
                    // let lastNPMInstallTime = moment(_projectInfo.LastNPMInstallTime, awsmobileJSConstant.DateTimeFormatString)
                    // let lastNodeModuleDirMTime = moment(dfops.getDirContentMTime(npm_modules_DirPath))
                    // let lastPackageJsonFileMTime = moment(fs.lstatSync(packageJsonFilePath).mtime)

                    // result =    !lastNPMInstallTime.isValid() ||
                    //             !lastNodeModuleDirMTime.isValid() ||
                    //             !lastPackageJsonFileMTime.isValid() ||
                    //             lastNPMInstallTime.isBefore(lastNodeModuleDirMTime) ||
                    //             lastNPMInstallTime.isBefore(lastPackageJsonFileMTime) 
                }
            }                  
        }
    }

    return result
}

function isNewBuildNeeded(){
    let result = false
    let distributionDirPath = path.normalize(path.join(_projectInfo.ProjectPath, _projectInfo.DistributionDir))

    if(fs.existsSync(distributionDirPath)){ 
        let ignoredDirs = [_projectInfo.DistributionDir, awsmobileJSConstant.AWSMobileJSDirName]
        let ignoredFiles = []

        let lastFrontendBuildTime = moment(_projectInfo.FrontendLastBuildTime, awsmobileJSConstant.DateTimeFormatString)
        let lastConfigurationTime = moment(_projectInfo.LastConfigurationTime, awsmobileJSConstant.DateTimeFormatString)
        let lastProjectModificationTime = moment(dfops.getDirContentMTime(_projectInfo.ProjectPath, ignoredDirs, ignoredFiles))
        let lastDistDirModificationTime = moment(dfops.getDirContentMTime(distributionDirPath))

        result =    !lastFrontendBuildTime.isValid() ||
                    !lastConfigurationTime.isValid() ||
                    !lastProjectModificationTime.isValid() ||
                    !lastDistDirModificationTime.isValid() ||
                    lastFrontendBuildTime.isBefore(lastConfigurationTime) ||
                    lastFrontendBuildTime.isBefore(lastProjectModificationTime) ||
                    lastFrontendBuildTime.isBefore(lastDistDirModificationTime) //user manually changed build
    }else{
        result = true
    }

    return result
}

module.exports = {
    build,
    checkNpmInstall
}

==> awsmobile-cli-1.1.6.tgz: package/lib/build-frontend.js <==
/* 
 * Copyright 2017-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 *
 *     http://aws.amazon.com/apache2.0/
 *
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
*/
"use strict";
const path = require('path')
const fs = require('fs-extra')
const ora = require('ora')
const moment = require('moment')
const chalk = require('chalk')

const { spawn } = require('child_process')

const projectInfoManager = require('./project-info-manager.js')
const pathManager = require('./utils/awsmobilejs-path-manager.js')
const awsmobilejsConstant = require('./utils/awsmobilejs-constant.js')
const dfops = require('./utils/directory-file-ops.js')
const opsProject = require('./backend-operations/ops-project.js')

let _projectInfo

function build(callback) {
    _projectInfo = projectInfoManager.getProjectInfo()
    if(_projectInfo){
        runNpmInstall(function(){
            runBuildCommand(callback)
        })
    }
}

function checkNpmInstall(callback){
    _projectInfo = projectInfoManager.getProjectInfo()
    if(_projectInfo){
        runNpmInstall(callback)
    }
}

function runNpmInstall(callback){
    if(isNpmInstallNeeded()){
        console.log()
        console.log('executing npm install ...')
        let npm = /^win/.test(process.platform) ? 'npm.cmd' : 'npm'
        let npmInstall = spawn(npm, ['install'], {cwd: process.cwd(), env: process.env, stdio: 'inherit'})
        npmInstall.on('exit', function(code) {
            console.log('npm install exited with code ' + code.toString())
            if(code == 0){
                _projectInfo.LastNPMInstallTime = moment().format(awsmobilejsConstant.DateTimeFormatString) 
                projectInfoManager.setProjectInfo(_projectInfo)
                if(callback){
                    callback()
                }
            }
        })
        npmInstall.on('error', (err) => {
            console.log(chalk.red('npm install execution error'))
            console.log(err)
        })
    }else if(callback){
        callback()
    }
}

function runBuildCommand(callback){
    if(isNewBuildNeeded() && _projectInfo.BuildCommand && _projectInfo.BuildCommand.length > 0){
        console.log()
        console.log('executing frontend build command ...')

        let args = _projectInfo.BuildCommand.split(/\s+/)
        let command = args[0]
        args = args.slice(1)
        
        let buildExecution = spawn(command, args, {cwd: process.cwd(), env: process.env, stdio: 'inherit'})
        buildExecution.on('exit', function(code) {
            console.log('build command exited with code ' + code.toString())
            if(code == 0){
                _projectInfo.FrontendLastBuildTime = moment().format(awsmobilejsConstant.DateTimeFormatString) 
                projectInfoManager.setProjectInfo(_projectInfo)
                if(callback){
                    callback()
                }
            }
        })
        buildExecution.on('error', (err) => {
            console.log(chalk.red('frontend build command execution error'))
            console.log(err)
        })
    }else{
        if(callback){
            callback()
        }
    }
}

function isNpmInstallNeeded(){
    let result = false
    
    let packageJsonFilePath = path.normalize(path.join(_projectInfo.ProjectPath, 'package.json'))
    if(fs.existsSync(packageJsonFilePath)){
        let npm_modules_DirPath = path.normalize(path.join(_projectInfo.ProjectPath, 'node_modules'))
        let packageInfo = JSON.parse(fs.readFileSync(packageJsonFilePath, 'utf8'))
        if(packageInfo){

            let hasDependency = (packageInfo.dependencies &&  Object.keys(packageInfo.dependencies).length>0) ||
                                (packageInfo.devDependencies &&  Object.keys(packageInfo.devDependencies).length>0)
            if(hasDependency){
                if(!fs.existsSync(npm_modules_DirPath)){
                    result = true
                }else{
                    // let lastNPMInstallTime = moment(_projectInfo.LastNPMInstallTime, awsmobilejsConstant.DateTimeFormatString)
                    // let lastNodeModuleDirMTime = moment(dfops.getDirContentMTime(npm_modules_DirPath))
                    // let lastPackageJsonFileMTime = moment(fs.lstatSync(packageJsonFilePath).mtime)

                    // result =    !lastNPMInstallTime.isValid() ||
                    //             !lastNodeModuleDirMTime.isValid() ||
                    //             !lastPackageJsonFileMTime.isValid() ||
                    //             lastNPMInstallTime.isBefore(lastNodeModuleDirMTime) ||
                    //             lastNPMInstallTime.isBefore(lastPackageJsonFileMTime) 
                }
            }                  
        }
    }

    return result
}

function isNewBuildNeeded(){
    let result = false
    let distributionDirPath = path.normalize(path.join(_projectInfo.ProjectPath, _projectInfo.DistributionDir))

    if(fs.existsSync(distributionDirPath)){ 
        let ignoredDirs = [_projectInfo.DistributionDir, awsmobilejsConstant.AWSMobileJSDirName]
        let ignoredFiles = []

        let lastFrontendBuildTime = moment(_projectInfo.FrontendLastBuildTime, awsmobilejsConstant.DateTimeFormatString)
        let lastConfigurationTime = moment(_projectInfo.LastConfigurationTime, awsmobilejsConstant.DateTimeFormatString)
        let lastProjectModificationTime = moment(dfops.getDirContentMTime(_projectInfo.ProjectPath, ignoredDirs, ignoredFiles))
        let lastDistDirModificationTime = moment(dfops.getDirContentMTime(distributionDirPath))

        result =    !lastFrontendBuildTime.isValid() ||
                    !lastConfigurationTime.isValid() ||
                    !lastProjectModificationTime.isValid() ||
                    !lastDistDirModificationTime.isValid() ||
                    lastFrontendBuildTime.isBefore(lastConfigurationTime) ||
                    lastFrontendBuildTime.isBefore(lastProjectModificationTime) ||
                    lastFrontendBuildTime.isBefore(lastDistDirModificationTime) //user manually changed build
    }else{
        result = true
    }

    return result
}

module.exports = {
    build,
    checkNpmInstall
}
